<?php

return [
    'apache_getenv',
    'apache_get_version',
    'apache_lookup_uri',
    'apache_setenv',
    'apcu_cache_info',
    'apcu_cas',
    'apcu_dec',
    'apcu_fetch',
    'apcu_inc',
    'apcu_sma_info',
    'base64_decode',
    'bindtextdomain',
    'bzclose',
    'bzflush',
    'bzopen',
    'bzread',
    'bzwrite',
    'chdir',
    'chgrp',
    'chmod',
    'chown',
    'chroot',
    'class_alias',
    'class_implements',
    'class_parents',
    'class_uses',
    'cli_set_process_title',
    'com_create_guid',
    'com_event_sink',
    'com_load_typelib',
    'com_print_typeinfo',
    'convert_uudecode',
    'copy',
    'cubrid_bind',
    'cubrid_column_names',
    'cubrid_column_types',
    'cubrid_col_size',
    'cubrid_commit',
    'cubrid_connect',
    'cubrid_connect_with_url',
    'cubrid_current_oid',
    'cubrid_disconnect',
    'cubrid_drop',
    'cubrid_free_result',
    'cubrid_get_charset',
    'cubrid_get_class_name',
    'cubrid_get_client_info',
    'cubrid_get_db_parameter',
    'cubrid_get_query_timeout',
    'cubrid_get_server_info',
    'cubrid_insert_id',
    'cubrid_lob2_bind',
    'cubrid_lob2_close',
    'cubrid_lob2_export',
    'cubrid_lob2_import',
    'cubrid_lob2_new',
    'cubrid_lob2_read',
    'cubrid_lob2_seek',
    'cubrid_lob2_seek64',
    'cubrid_lob2_size',
    'cubrid_lob2_size64',
    'cubrid_lob2_tell',
    'cubrid_lob2_tell64',
    'cubrid_lob2_write',
    'cubrid_lob_close',
    'cubrid_lob_export',
    'cubrid_lob_get',
    'cubrid_lob_send',
    'cubrid_lob_size',
    'cubrid_lock_read',
    'cubrid_lock_write',
    'cubrid_move_cursor',
    'cubrid_next_result',
    'cubrid_pconnect',
    'cubrid_pconnect_with_url',
    'cubrid_prepare',
    'cubrid_put',
    'cubrid_rollback',
    'cubrid_schema',
    'cubrid_seq_drop',
    'cubrid_seq_insert',
    'cubrid_seq_put',
    'cubrid_set_add',
    'cubrid_set_autocommit',
    'cubrid_set_db_parameter',
    'cubrid_set_drop',
    'cubrid_set_query_timeout',
    'curl_copy_handle',
    'curl_escape',
    'curl_exec',
    'curl_getinfo',
    'curl_init',
    'curl_multi_info_read',
    'curl_multi_setopt',
    'curl_setopt',
    'curl_share_setopt',
    'curl_unescape',
    'curl_upkeep',
    'date_create',
    'date_create_immutable',
    'date_parse_from_format',
    'date_sunrise',
    'date_sunset',
    'db2_autocommit',
    'db2_bind_param',
    'db2_client_info',
    'db2_close',
    'db2_commit',
    'db2_execute',
    'db2_free_result',
    'db2_free_stmt',
    'db2_get_option',
    'db2_num_rows',
    'db2_pclose',
    'db2_rollback',
    'db2_server_info',
    'db2_set_option',
    'define',
    'deflate_add',
    'deflate_init',
    'dir',
    'disk_free_space',
    'disk_total_space',
    'dl',
    'dns_get_record',
    'eio_busy',
    'eio_chmod',
    'eio_chown',
    'eio_close',
    'eio_custom',
    'eio_dup2',
    'eio_event_loop',
    'eio_fallocate',
    'eio_fchmod',
    'eio_fchown',
    'eio_fdatasync',
    'eio_fstat',
    'eio_fstatvfs',
    'eio_fsync',
    'eio_ftruncate',
    'eio_futime',
    'eio_grp',
    'eio_lstat',
    'eio_mkdir',
    'eio_mknod',
    'eio_nop',
    'eio_readahead',
    'eio_readdir',
    'eio_readlink',
    'eio_rename',
    'eio_rmdir',
    'eio_seek',
    'eio_sendfile',
    'eio_stat',
    'eio_statvfs',
    'eio_symlink',
    'eio_sync',
    'eio_syncfs',
    'eio_sync_file_range',
    'eio_truncate',
    'eio_unlink',
    'eio_utime',
    'eio_write',
    'error_log',
    'exec',
    'fastcgi_finish_request',
    'fbird_blob_cancel',
    'fclose',
    'fdatasync',
    'fflush',
    'fgetcsv',
    'file',
    'fileatime',
    'filectime',
    'fileinode',
    'filemtime',
    'fileowner',
    'fileperms',
    'filesize',
    'filetype',
    'file_get_contents',
    'file_put_contents',
    'filter_input_array',
    'filter_var_array',
    'finfo_close',
    'finfo_open',
    'flock',
    'fopen',
    'fpm_get_status',
    'fputcsv',
    'fread',
    'fsockopen',
    'fstat',
    'fsync',
    'ftell',
    'ftp_alloc',
    'ftp_append',
    'ftp_cdup',
    'ftp_chdir',
    'ftp_chmod',
    'ftp_close',
    'ftp_connect',
    'ftp_delete',
    'ftp_fget',
    'ftp_fput',
    'ftp_get',
    'ftp_login',
    'ftp_mkdir',
    'ftp_mlsd',
    'ftp_nb_get',
    'ftp_nb_put',
    'ftp_nlist',
    'ftp_pasv',
    'ftp_put',
    'ftp_pwd',
    'ftp_raw',
    'ftp_rename',
    'ftp_rmdir',
    'ftp_site',
    'ftp_size',
    'ftp_ssl_connect',
    'ftp_systype',
    'ftruncate',
    'fwrite',
    'getcwd',
    'gethostname',
    'getimagesize',
    'getlastmod',
    'getmygid',
    'getmyinode',
    'getmypid',
    'getmyuid',
    'getopt',
    'getprotobyname',
    'getprotobynumber',
    'getrusage',
    'getservbyport',
    'get_cfg_var',
    'get_headers',
    'get_include_path',
    'get_meta_tags',
    'glob',
    'gmmktime',
    'gmstrftime',
    'gnupg_adddecryptkey',
    'gnupg_addencryptkey',
    'gnupg_addsignkey',
    'gnupg_cleardecryptkeys',
    'gnupg_clearencryptkeys',
    'gnupg_clearsignkeys',
    'gnupg_deletekey',
    'gnupg_setarmor',
    'gnupg_setsignmode',
    'gzclose',
    'gzcompress',
    'gzdecode',
    'gzdeflate',
    'gzencode',
    'gzfile',
    'gzgets',
    'gzinflate',
    'gzopen',
    'gzread',
    'gzrewind',
    'gztell',
    'gzuncompress',
    'gzwrite',
    'hash_file',
    'hash_hmac_file',
    'hash_update_file',
    'header_register_callback',
    'hex2bin',
    'highlight_file',
    'hrtime',
    'ibase_add_user',
    'ibase_backup',
    'ibase_blob_cancel',
    'ibase_blob_create',
    'ibase_blob_get',
    'ibase_close',
    'ibase_commit',
    'ibase_commit_ret',
    'ibase_connect',
    'ibase_delete_user',
    'ibase_drop_db',
    'ibase_free_event_handler',
    'ibase_free_query',
    'ibase_free_result',
    'ibase_maintain_db',
    'ibase_modify_user',
    'ibase_name_result',
    'ibase_pconnect',
    'ibase_restore',
    'ibase_rollback',
    'ibase_rollback_ret',
    'ibase_service_attach',
    'ibase_service_detach',
    'iconv',
    'iconv_get_encoding',
    'iconv_mime_decode',
    'iconv_mime_encode',
    'iconv_set_encoding',
    'iconv_strlen',
    'idate',
    'imageaffine',
    'imageaffinematrixconcat',
    'imageaffinematrixget',
    'imagealphablending',
    'imageantialias',
    'imagearc',
    'imageavif',
    'imagebmp',
    'imagechar',
    'imagecharup',
    'imagecolorat',
    'imagecolordeallocate',
    'imagecolormatch',
    'imagecolorset',
    'imageconvolution',
    'imagecopy',
    'imagecopymerge',
    'imagecopymergegray',
    'imagecopyresampled',
    'imagecopyresized',
    'imagecreate',
    'imagecreatefromavif',
    'imagecreatefrombmp',
    'imagecreatefromgd',
    'imagecreatefromgd2',
    'imagecreatefromgd2part',
    'imagecreatefromgif',
    'imagecreatefromjpeg',
    'imagecreatefrompng',
    'imagecreatefromstring',
    'imagecreatefromtga',
    'imagecreatefromwbmp',
    'imagecreatefromwebp',
    'imagecreatefromxbm',
    'imagecreatefromxpm',
    'imagecreatetruecolor',
    'imagecrop',
    'imagecropauto',
    'imagedashedline',
    'imagedestroy',
    'imageellipse',
    'imagefill',
    'imagefilledarc',
    'imagefilledellipse',
    'imagefilledrectangle',
    'imagefilltoborder',
    'imagefilter',
    'imageflip',
    'imageftbbox',
    'imagefttext',
    'imagegammacorrect',
    'imagegd',
    'imagegd2',
    'imagegif',
    'imagegrabscreen',
    'imagegrabwindow',
    'imagejpeg',
    'imagelayereffect',
    'imageline',
    'imageloadfont',
    'imagepng',
    'imagerectangle',
    'imageresolution',
    'imagerotate',
    'imagesavealpha',
    'imagescale',
    'imagesetbrush',
    'imagesetclip',
    'imagesetinterpolation',
    'imagesetpixel',
    'imagesetstyle',
    'imagesetthickness',
    'imagesettile',
    'imagestring',
    'imagestringup',
    'imagetruecolortopalette',
    'imagettfbbox',
    'imagettftext',
    'imagewbmp',
    'imagewebp',
    'imagexbm',
    'image_type_to_extension',
    'imap_8bit',
    'imap_append',
    'imap_base64',
    'imap_binary',
    'imap_body',
    'imap_bodystruct',
    'imap_check',
    'imap_createmailbox',
    'imap_deletemailbox',
    'imap_fetchbody',
    'imap_fetchheader',
    'imap_fetchmime',
    'imap_fetchstructure',
    'imap_fetch_overview',
    'imap_getacl',
    'imap_getmailboxes',
    'imap_getsubscribed',
    'imap_headerinfo',
    'imap_headers',
    'imap_listscan',
    'imap_lsub',
    'imap_mail',
    'imap_mail_compose',
    'imap_mail_copy',
    'imap_mail_move',
    'imap_mime_header_decode',
    'imap_mutf7_to_utf8',
    'imap_num_msg',
    'imap_open',
    'imap_qprint',
    'imap_renamemailbox',
    'imap_rfc822_write_address',
    'imap_savebody',
    'imap_setacl',
    'imap_set_quota',
    'imap_sort',
    'imap_status',
    'imap_subscribe',
    'imap_thread',
    'imap_timeout',
    'imap_unsubscribe',
    'imap_utf8_to_mutf7',
    'inet_ntop',
    'inet_pton',
    'inflate_add',
    'inflate_get_read_len',
    'inflate_init',
    'ini_get',
    'ini_set',
    'inotify_add_watch',
    'inotify_init',
    'inotify_rm_watch',
    'iptcembed',
    'iptcparse',
    'json_decode',
    'json_encode',
    'lchgrp',
    'lchown',
    'ldap_8859_to_t61',
    'ldap_add',
    'ldap_bind',
    'ldap_compare',
    'ldap_control_paged_result',
    'ldap_control_paged_result_response',
    'ldap_count_entries',
    'ldap_delete',
    'ldap_dn2ufn',
    'ldap_exop',
    'ldap_exop_passwd',
    'ldap_exop_whoami',
    'ldap_explode_dn',
    'ldap_first_attribute',
    'ldap_first_entry',
    'ldap_free_result',
    'ldap_get_dn',
    'ldap_get_entries',
    'ldap_get_option',
    'ldap_get_values',
    'ldap_get_values_len',
    'ldap_modify_batch',
    'ldap_mod_add',
    'ldap_mod_del',
    'ldap_mod_replace',
    'ldap_next_attribute',
    'ldap_parse_exop',
    'ldap_parse_result',
    'ldap_rename',
    'ldap_sasl_bind',
    'ldap_set_option',
    'ldap_unbind',
    'libxml_set_external_entity_loader',
    'link',
    'lstat',
    'lzf_compress',
    'lzf_decompress',
    'mailparse_msg_extract_part_file',
    'mailparse_msg_free',
    'mailparse_msg_parse',
    'mailparse_msg_parse_file',
    'mailparse_stream_encode',
    'mb_chr',
    'mb_convert_encoding',
    'mb_convert_variables',
    'mb_detect_order',
    'mb_eregi_replace',
    'mb_ereg_replace',
    'mb_ereg_replace_callback',
    'mb_ereg_search_getregs',
    'mb_ereg_search_init',
    'mb_ereg_search_regs',
    'mb_ereg_search_setpos',
    'mb_get_info',
    'mb_http_output',
    'mb_internal_encoding',
    'mb_ord',
    'mb_parse_str',
    'mb_regex_encoding',
    'mb_send_mail',
    'mb_split',
    'md5_file',
    'mime_content_type',
    'mkdir',
    'mktime',
    'msg_get_queue',
    'msg_queue_exists',
    'msg_receive',
    'msg_remove_queue',
    'msg_send',
    'msg_set_queue',
    'msg_stat_queue',
    'mysql_close',
    'mysql_connect',
    'mysql_create_db',
    'mysql_data_seek',
    'mysql_db_name',
    'mysql_db_query',
    'mysql_drop_db',
    'mysql_fetch_lengths',
    'mysql_field_flags',
    'mysql_field_len',
    'mysql_field_name',
    'mysql_field_seek',
    'mysql_free_result',
    'mysql_get_host_info',
    'mysql_get_proto_info',
    'mysql_get_server_info',
    'mysql_info',
    'mysql_list_dbs',
    'mysql_list_fields',
    'mysql_list_processes',
    'mysql_list_tables',
    'mysql_num_fields',
    'mysql_num_rows',
    'mysql_query',
    'mysql_real_escape_string',
    'mysql_result',
    'mysql_select_db',
    'mysql_set_charset',
    'mysql_tablename',
    'mysql_thread_id',
    'mysql_unbuffered_query',
    'net_get_interfaces',
    'ob_clean',
    'ob_end_clean',
    'ob_end_flush',
    'ob_flush',
    'ob_get_clean',
    'ob_get_flush',
    'ob_start',
    'oci_bind_array_by_name',
    'oci_bind_by_name',
    'oci_cancel',
    'oci_commit',
    'oci_connect',
    'oci_define_by_name',
    'oci_execute',
    'oci_field_name',
    'oci_field_precision',
    'oci_field_scale',
    'oci_field_size',
    'oci_field_type',
    'oci_field_type_raw',
    'oci_free_descriptor',
    'oci_free_statement',
    'oci_new_collection',
    'oci_new_connect',
    'oci_new_cursor',
    'oci_new_descriptor',
    'oci_num_rows',
    'oci_parse',
    'oci_pconnect',
    'oci_register_taf_callback',
    'oci_result',
    'oci_rollback',
    'oci_server_version',
    'oci_set_action',
    'oci_set_call_timeout',
    'oci_set_client_identifier',
    'oci_set_client_info',
    'oci_set_db_operation',
    'oci_set_edition',
    'oci_set_module_name',
    'oci_set_prefetch',
    'oci_set_prefetch_lob',
    'oci_statement_type',
    'oci_unregister_taf_callback',
    'odbc_autocommit',
    'odbc_commit',
    'odbc_connect',
    'odbc_cursor',
    'odbc_data_source',
    'odbc_exec',
    'odbc_execute',
    'odbc_fetch_into',
    'odbc_field_len',
    'odbc_field_name',
    'odbc_field_num',
    'odbc_field_scale',
    'odbc_field_type',
    'odbc_num_fields',
    'odbc_pconnect',
    'odbc_prepare',
    'odbc_procedures',
    'odbc_result',
    'odbc_result_all',
    'odbc_rollback',
    'odbc_setoption',
    'odbc_tables',
    'opcache_compile_file',
    'opcache_get_status',
    'opendir',
    'openssl_cipher_iv_length',
    'openssl_cipher_key_length',
    'openssl_cms_decrypt',
    'openssl_cms_encrypt',
    'openssl_cms_read',
    'openssl_cms_sign',
    'openssl_cms_verify',
    'openssl_csr_export',
    'openssl_csr_export_to_file',
    'openssl_csr_get_public_key',
    'openssl_csr_get_subject',
    'openssl_csr_new',
    'openssl_csr_sign',
    'openssl_decrypt',
    'openssl_dh_compute_key',
    'openssl_digest',
    'openssl_encrypt',
    'openssl_get_curve_names',
    'openssl_open',
    'openssl_pbkdf2',
    'openssl_pkcs7_decrypt',
    'openssl_pkcs7_encrypt',
    'openssl_pkcs7_read',
    'openssl_pkcs7_sign',
    'openssl_pkcs12_export',
    'openssl_pkcs12_export_to_file',
    'openssl_pkcs12_read',
    'openssl_pkey_derive',
    'openssl_pkey_export',
    'openssl_pkey_export_to_file',
    'openssl_pkey_get_details',
    'openssl_pkey_get_private',
    'openssl_pkey_get_public',
    'openssl_pkey_new',
    'openssl_private_decrypt',
    'openssl_private_encrypt',
    'openssl_public_decrypt',
    'openssl_public_encrypt',
    'openssl_seal',
    'openssl_sign',
    'openssl_spki_export',
    'openssl_spki_export_challenge',
    'openssl_spki_new',
    'openssl_spki_verify',
    'openssl_verify',
    'openssl_x509_checkpurpose',
    'openssl_x509_export',
    'openssl_x509_export_to_file',
    'openssl_x509_fingerprint',
    'openssl_x509_read',
    'output_add_rewrite_var',
    'output_reset_rewrite_vars',
    'parse_ini_file',
    'parse_ini_string',
    'parse_url',
    'passthru',
    'pclose',
    'pcntl_getpriority',
    'pcntl_setpriority',
    'pcntl_signal',
    'pcntl_signal_dispatch',
    'pcntl_sigprocmask',
    'pcntl_sigtimedwait',
    'pcntl_sigwaitinfo',
    'pfsockopen',
    'pg_cancel_query',
    'pg_connect',
    'pg_connection_reset',
    'pg_convert',
    'pg_copy_from',
    'pg_copy_to',
    'pg_delete',
    'pg_end_copy',
    'pg_execute',
    'pg_field_num',
    'pg_field_table',
    'pg_flush',
    'pg_free_result',
    'pg_host',
    'pg_insert',
    'pg_last_oid',
    'pg_lo_close',
    'pg_lo_export',
    'pg_lo_import',
    'pg_lo_open',
    'pg_lo_read',
    'pg_lo_seek',
    'pg_lo_truncate',
    'pg_lo_unlink',
    'pg_lo_write',
    'pg_meta_data',
    'pg_parameter_status',
    'pg_pconnect',
    'pg_ping',
    'pg_prepare',
    'pg_put_line',
    'pg_query',
    'pg_query_params',
    'pg_result_error_field',
    'pg_result_seek',
    'pg_select',
    'pg_set_chunked_rows_size',
    'pg_socket',
    'pg_trace',
    'pg_update',
    'php_sapi_name',
    'popen',
    'posix_access',
    'posix_eaccess',
    'posix_getgrgid',
    'posix_getgrnam',
    'posix_getgroups',
    'posix_getlogin',
    'posix_getpgid',
    'posix_getpwuid',
    'posix_getrlimit',
    'posix_getsid',
    'posix_initgroups',
    'posix_kill',
    'posix_mkfifo',
    'posix_mknod',
    'posix_setegid',
    'posix_seteuid',
    'posix_setgid',
    'posix_setpgid',
    'posix_setrlimit',
    'posix_setsid',
    'posix_setuid',
    'posix_times',
    'posix_uname',
    'preg_grep',
    'preg_match',
    'preg_match_all',
    'preg_replace',
    'preg_replace_callback',
    'preg_replace_callback_array',
    'preg_split',
    'proc_close',
    'proc_nice',
    'proc_open',
    'pspell_add_to_personal',
    'pspell_add_to_session',
    'pspell_clear_session',
    'pspell_config_data_dir',
    'pspell_config_dict_dir',
    'pspell_config_ignore',
    'pspell_config_mode',
    'pspell_config_personal',
    'pspell_config_repl',
    'pspell_config_runtogether',
    'pspell_config_save_repl',
    'pspell_new',
    'pspell_new_config',
    'pspell_new_personal',
    'pspell_save_wordlist',
    'pspell_store_replacement',
    'ps_add_launchlink',
    'ps_add_locallink',
    'ps_add_note',
    'ps_add_pdflink',
    'ps_add_weblink',
    'ps_arc',
    'ps_arcn',
    'ps_begin_page',
    'ps_begin_pattern',
    'ps_begin_template',
    'ps_circle',
    'ps_clip',
    'ps_close',
    'ps_closepath',
    'ps_closepath_stroke',
    'ps_close_image',
    'ps_continue_text',
    'ps_curveto',
    'ps_delete',
    'ps_end_page',
    'ps_end_pattern',
    'ps_end_template',
    'ps_fill',
    'ps_fill_stroke',
    'ps_get_parameter',
    'ps_hyphenate',
    'ps_include_file',
    'ps_lineto',
    'ps_moveto',
    'ps_new',
    'ps_open_file',
    'ps_place_image',
    'ps_rect',
    'ps_restore',
    'ps_rotate',
    'ps_save',
    'ps_scale',
    'ps_setcolor',
    'ps_setdash',
    'ps_setflat',
    'ps_setfont',
    'ps_setgray',
    'ps_setlinecap',
    'ps_setlinejoin',
    'ps_setlinewidth',
    'ps_setmiterlimit',
    'ps_setoverprintmode',
    'ps_setpolydash',
    'ps_set_border_color',
    'ps_set_border_dash',
    'ps_set_border_style',
    'ps_set_info',
    'ps_set_parameter',
    'ps_set_text_pos',
    'ps_set_value',
    'ps_shading',
    'ps_shading_pattern',
    'ps_shfill',
    'ps_show',
    'ps_show2',
    'ps_show_xy',
    'ps_show_xy2',
    'ps_stroke',
    'ps_symbol',
    'ps_translate',
    'putenv',
    'readfile',
    'readgzfile',
    'readline_completion_function',
    'readline_read_history',
    'readline_write_history',
    'readlink',
    'realpath',
    'register_tick_function',
    'rename',
    'rewind',
    'rmdir',
    'rnp_decrypt',
    'rnp_dump_packets',
    'rnp_dump_packets_to_json',
    'rnp_ffi_create',
    'rnp_ffi_set_pass_provider',
    'rnp_import_keys',
    'rnp_import_signatures',
    'rnp_key_export',
    'rnp_key_export_autocrypt',
    'rnp_key_export_revocation',
    'rnp_key_get_info',
    'rnp_key_remove',
    'rnp_key_revoke',
    'rnp_list_keys',
    'rnp_load_keys',
    'rnp_load_keys_from_path',
    'rnp_locate_key',
    'rnp_op_encrypt',
    'rnp_op_generate_key',
    'rnp_op_sign',
    'rnp_op_sign_cleartext',
    'rnp_op_sign_detached',
    'rnp_op_verify',
    'rnp_op_verify_detached',
    'rnp_save_keys',
    'rnp_save_keys_to_path',
    'rnp_supported_features',
    'rpmaddtag',
    'rrd_create',
    'rrd_first',
    'rrd_graph',
    'rrd_info',
    'rrd_lastupdate',
    'rrd_restore',
    'rrd_tune',
    'rrd_update',
    'rrd_xport',
    'sapi_windows_cp_conv',
    'sapi_windows_cp_set',
    'sapi_windows_generate_ctrl_event',
    'sapi_windows_set_ctrl_handler',
    'sapi_windows_vt100_support',
    'scandir',
    'sem_acquire',
    'sem_get',
    'sem_release',
    'sem_remove',
    'session_abort',
    'session_cache_expire',
    'session_cache_limiter',
    'session_create_id',
    'session_decode',
    'session_destroy',
    'session_encode',
    'session_gc',
    'session_id',
    'session_module_name',
    'session_name',
    'session_regenerate_id',
    'session_reset',
    'session_save_path',
    'session_start',
    'session_unset',
    'session_write_close',
    'settype',
    'set_include_path',
    'set_time_limit',
    'sha1_file',
    'shell_exec',
    'shmop_delete',
    'shmop_read',
    'shm_attach',
    'shm_detach',
    'shm_put_var',
    'shm_remove',
    'shm_remove_var',
    'simplexml_import_dom',
    'simplexml_load_file',
    'simplexml_load_string',
    'socket_accept',
    'socket_addrinfo_bind',
    'socket_addrinfo_connect',
    'socket_addrinfo_lookup',
    'socket_atmark',
    'socket_bind',
    'socket_connect',
    'socket_create',
    'socket_create_listen',
    'socket_create_pair',
    'socket_export_stream',
    'socket_getpeername',
    'socket_getsockname',
    'socket_get_option',
    'socket_import_stream',
    'socket_listen',
    'socket_read',
    'socket_send',
    'socket_sendmsg',
    'socket_sendto',
    'socket_set_block',
    'socket_set_nonblock',
    'socket_set_option',
    'socket_shutdown',
    'socket_write',
    'socket_wsaprotocol_info_export',
    'socket_wsaprotocol_info_import',
    'socket_wsaprotocol_info_release',
    'sodium_crypto_aead_aegis128l_decrypt',
    'sodium_crypto_aead_aegis256_decrypt',
    'sodium_crypto_aead_aes256gcm_decrypt',
    'sodium_crypto_aead_chacha20poly1305_decrypt',
    'sodium_crypto_aead_chacha20poly1305_encrypt',
    'sodium_crypto_aead_chacha20poly1305_ietf_decrypt',
    'sodium_crypto_aead_chacha20poly1305_ietf_encrypt',
    'sodium_crypto_aead_xchacha20poly1305_ietf_decrypt',
    'sodium_crypto_aead_xchacha20poly1305_ietf_encrypt',
    'sodium_crypto_box_open',
    'sodium_crypto_box_seal_open',
    'sodium_crypto_secretbox_open',
    'sodium_crypto_sign_open',
    'sodium_crypto_stream_xchacha20_xor_ic',
    'solr_get_version',
    'spl_autoload_register',
    'spl_autoload_unregister',
    'sqlsrv_begin_transaction',
    'sqlsrv_cancel',
    'sqlsrv_client_info',
    'sqlsrv_close',
    'sqlsrv_commit',
    'sqlsrv_configure',
    'sqlsrv_execute',
    'sqlsrv_fetch',
    'sqlsrv_fetch_array',
    'sqlsrv_fetch_object',
    'sqlsrv_free_stmt',
    'sqlsrv_get_field',
    'sqlsrv_next_result',
    'sqlsrv_num_fields',
    'sqlsrv_num_rows',
    'sqlsrv_prepare',
    'sqlsrv_query',
    'sqlsrv_rollback',
    'ssdeep_fuzzy_compare',
    'ssdeep_fuzzy_hash',
    'ssdeep_fuzzy_hash_filename',
    'ssh2_auth_agent',
    'ssh2_auth_hostbased_file',
    'ssh2_auth_password',
    'ssh2_auth_pubkey_file',
    'ssh2_connect',
    'ssh2_disconnect',
    'ssh2_exec',
    'ssh2_forward_accept',
    'ssh2_forward_listen',
    'ssh2_publickey_add',
    'ssh2_publickey_init',
    'ssh2_publickey_remove',
    'ssh2_scp_recv',
    'ssh2_scp_send',
    'ssh2_send_eof',
    'ssh2_sftp',
    'ssh2_sftp_chmod',
    'ssh2_sftp_mkdir',
    'ssh2_sftp_rename',
    'ssh2_sftp_rmdir',
    'ssh2_sftp_symlink',
    'ssh2_sftp_unlink',
    'ssh2_shell',
    'stream_context_set_options',
    'stream_context_set_params',
    'stream_copy_to_stream',
    'stream_filter_append',
    'stream_filter_prepend',
    'stream_filter_register',
    'stream_filter_remove',
    'stream_get_contents',
    'stream_get_line',
    'stream_isatty',
    'stream_resolve_include_path',
    'stream_set_blocking',
    'stream_set_timeout',
    'stream_socket_accept',
    'stream_socket_client',
    'stream_socket_get_name',
    'stream_socket_pair',
    'stream_socket_recvfrom',
    'stream_socket_sendto',
    'stream_socket_server',
    'stream_socket_shutdown',
    'stream_supports_lock',
    'stream_wrapper_register',
    'stream_wrapper_restore',
    'stream_wrapper_unregister',
    'strftime',
    'strptime',
    'strtotime',
    'swoole_async_dns_lookup',
    'swoole_async_readfile',
    'swoole_async_write',
    'swoole_async_writefile',
    'swoole_event_defer',
    'swoole_event_del',
    'swoole_event_write',
    'symlink',
    'system',
    'sys_getloadavg',
    'tempnam',
    'timezone_name_from_abbr',
    'time_nanosleep',
    'time_sleep_until',
    'tmpfile',
    'touch',
    'unixtojd',
    'unlink',
    'unpack',
    'unserialize',
    'uopz_extend',
    'uopz_implement',
    'variant_date_to_timestamp',
    'variant_round',
    'virtual',
    'xdiff_file_bdiff',
    'xdiff_file_bpatch',
    'xdiff_file_diff',
    'xdiff_file_diff_binary',
    'xdiff_file_patch_binary',
    'xdiff_file_rabdiff',
    'xdiff_string_bpatch',
    'xdiff_string_patch',
    'xdiff_string_patch_binary',
    'xmlrpc_set_type',
    'xml_parser_free',
    'xml_parser_set_option',
    'yaml_parse',
    'yaml_parse_file',
    'yaml_parse_url',
    'yaz_ccl_parse',
    'yaz_close',
    'yaz_connect',
    'yaz_database',
    'yaz_element',
    'yaz_present',
    'yaz_search',
    'yaz_wait',
    'zip_entry_close',
    'zip_entry_compressedsize',
    'zip_entry_compressionmethod',
    'zip_entry_filesize',
    'zip_entry_name',
    'zip_entry_open',
    'zip_entry_read',
    'zlib_decode',
];
